/* DO NOT EDIT THIS FILE - it is machine generated */

#ifndef __gnu_java_awt_peer_gtk_GdkFontMetrics__
#define __gnu_java_awt_peer_gtk_GdkFontMetrics__

#include <jni.h>

#ifdef __cplusplus
extern "C"
{
#endif

JNIEXPORT void JNICALL Java_gnu_java_awt_peer_gtk_GdkFontMetrics_getPeerFontMetrics (JNIEnv *env, jclass, jobject, jdoubleArray);
JNIEXPORT void JNICALL Java_gnu_java_awt_peer_gtk_GdkFontMetrics_getPeerTextMetrics (JNIEnv *env, jclass, jobject, jstring, jdoubleArray);
#undef gnu_java_awt_peer_gtk_GdkFontMetrics_FONT_METRICS_ASCENT
#define gnu_java_awt_peer_gtk_GdkFontMetrics_FONT_METRICS_ASCENT 0L
#undef gnu_java_awt_peer_gtk_GdkFontMetrics_FONT_METRICS_MAX_ASCENT
#define gnu_java_awt_peer_gtk_GdkFontMetrics_FONT_METRICS_MAX_ASCENT 1L
#undef gnu_java_awt_peer_gtk_GdkFontMetrics_FONT_METRICS_DESCENT
#define gnu_java_awt_peer_gtk_GdkFontMetrics_FONT_METRICS_DESCENT 2L
#undef gnu_java_awt_peer_gtk_GdkFontMetrics_FONT_METRICS_MAX_DESCENT
#define gnu_java_awt_peer_gtk_GdkFontMetrics_FONT_METRICS_MAX_DESCENT 3L
#undef gnu_java_awt_peer_gtk_GdkFontMetrics_FONT_METRICS_MAX_ADVANCE
#define gnu_java_awt_peer_gtk_GdkFontMetrics_FONT_METRICS_MAX_ADVANCE 4L
#undef gnu_java_awt_peer_gtk_GdkFontMetrics_TEXT_METRICS_X_BEARING
#define gnu_java_awt_peer_gtk_GdkFontMetrics_TEXT_METRICS_X_BEARING 0L
#undef gnu_java_awt_peer_gtk_GdkFontMetrics_TEXT_METRICS_Y_BEARING
#define gnu_java_awt_peer_gtk_GdkFontMetrics_TEXT_METRICS_Y_BEARING 1L
#undef gnu_java_awt_peer_gtk_GdkFontMetrics_TEXT_METRICS_WIDTH
#define gnu_java_awt_peer_gtk_GdkFontMetrics_TEXT_METRICS_WIDTH 2L
#undef gnu_java_awt_peer_gtk_GdkFontMetrics_TEXT_METRICS_HEIGHT
#define gnu_java_awt_peer_gtk_GdkFontMetrics_TEXT_METRICS_HEIGHT 3L
#undef gnu_java_awt_peer_gtk_GdkFontMetrics_TEXT_METRICS_X_ADVANCE
#define gnu_java_awt_peer_gtk_GdkFontMetrics_TEXT_METRICS_X_ADVANCE 4L
#undef gnu_java_awt_peer_gtk_GdkFontMetrics_TEXT_METRICS_Y_ADVANCE
#define gnu_java_awt_peer_gtk_GdkFontMetrics_TEXT_METRICS_Y_ADVANCE 5L

#ifdef __cplusplus
}
#endif

#endif /* __gnu_java_awt_peer_gtk_GdkFontMetrics__ */
