/* DO NOT EDIT THIS FILE - it is machine generated */

#ifndef __gnu_java_nio_VMPipe__
#define __gnu_java_nio_VMPipe__

#include <jni.h>

#ifdef __cplusplus
extern "C"
{
#endif

JNIEXPORT void JNICALL Java_gnu_java_nio_VMPipe_init (JNIEnv *env, jclass, jobject, jobject);

#ifdef __cplusplus
}
#endif

#endif /* __gnu_java_nio_VMPipe__ */
