/* DO NOT EDIT THIS FILE - it is machine generated */

#ifndef __java_nio_DirectByteBufferImpl__
#define __java_nio_DirectByteBufferImpl__

#include <jni.h>

#ifdef __cplusplus
extern "C"
{
#endif

JNIEXPORT jobject JNICALL Java_java_nio_DirectByteBufferImpl_allocateImpl (JNIEnv *env, jclass, jint);
JNIEXPORT void JNICALL Java_java_nio_DirectByteBufferImpl_freeImpl (JNIEnv *env, jclass, jobject);
JNIEXPORT jbyte JNICALL Java_java_nio_DirectByteBufferImpl_getImpl__Lgnu_classpath_RawData_2I (JNIEnv *env, jclass, jobject, jint);
JNIEXPORT void JNICALL Java_java_nio_DirectByteBufferImpl_putImpl (JNIEnv *env, jclass, jobject, jint, jbyte);
JNIEXPORT void JNICALL Java_java_nio_DirectByteBufferImpl_getImpl__Lgnu_classpath_RawData_2I_3BII (JNIEnv *env, jclass, jobject, jint, jbyteArray, jint, jint);
JNIEXPORT void JNICALL Java_java_nio_DirectByteBufferImpl_shiftDown (JNIEnv *env, jclass, jobject, jint, jint, jint);
JNIEXPORT jobject JNICALL Java_java_nio_DirectByteBufferImpl_adjustAddress (JNIEnv *env, jclass, jobject, jint);

#ifdef __cplusplus
}
#endif

#endif /* __java_nio_DirectByteBufferImpl__ */
